@@ -144,12 +144,19 @@ xfce_workspace_set_xfconf_property_string(XfceWorkspace *workspace,
 {
     XfconfChannel *channel = workspace->priv->channel;
     char buf[1024];
+#if 0 /* GTK_CHECK_VERSION (3, 22, 0) */
+    GdkDisplay *display;
+#endif
     gchar *monitor_name = NULL;
 
     TRACE("entering");
 
-    monitor_name = gdk_screen_get_monitor_plug_name(workspace->priv->gscreen,
-                                                    monitor_num);
+#if 0 /* GTK_CHECK_VERSION (3, 22, 0) */
+    display = gdk_display_manager_get_default_display(gdk_display_manager_get());
+    monitor_name = g_strdup(gdk_monitor_get_model(gdk_display_get_monitor(display, monitor_num)));
+#else
+    monitor_name = gdk_screen_get_monitor_plug_name(workspace->priv->gscreen, monitor_num);
+#endif
 
     /* Get the backdrop's image property */
     if(monitor_name == NULL) {
@@ -176,14 +183,21 @@ xfce_workspace_set_xfconf_property_value(XfceWorkspace *workspace,
     XfconfChannel *channel = workspace->priv->channel;
     char buf[1024];
     gchar *monitor_name = NULL;
+#if 0 /* GTK_CHECK_VERSION (3, 22, 0) */
+    GdkDisplay *display;
+#endif
 #ifdef G_ENABLE_DEBUG
     gchar *contents = NULL;
 #endif
 
     TRACE("entering");
 
-    monitor_name = gdk_screen_get_monitor_plug_name(workspace->priv->gscreen,
-                                                    monitor_num);
+#if 0 /* GTK_CHECK_VERSION (3, 22, 0) */
+    display = gdk_display_manager_get_default_display(gdk_display_manager_get());
+    monitor_name = g_strdup(gdk_monitor_get_model(gdk_display_get_monitor(display, monitor_num)));
+#else
+    monitor_name = gdk_screen_get_monitor_plug_name(workspace->priv->gscreen, monitor_num);
+#endif
 
     /* Get the backdrop's image property */
     if(monitor_name == NULL) {
@@ -305,7 +319,11 @@ xfce_workspace_monitors_changed(XfceWorkspace *workspace,
         /* When spanning screens we only need one backdrop */
         n_monitors = 1;
     } else {
-        n_monitors = gdk_screen_get_n_monitors(gscreen);
+#if 0 /* GTK_CHECK_VERSION (3, 22, 0) */
+        n_monitors = gdk_display_get_n_monitors(gdk_screen_get_display(workspace->priv->gscreen));
+#else
+        n_monitors = gdk_screen_get_n_monitors(workspace->priv->gscreen);
+#endif
     }
 
     /* Remove all backdrops so that the correct monitor is added/removed and
@@ -455,6 +473,12 @@ xfce_workspace_migrate_backdrop_first_color(XfceWorkspace *workspace,
 
     TRACE("entering");
 
+    /* TODO: migrate from color1 (and older) to the new rgba1 */
+    if(TRUE) {
+        TRACE("warning: we aren't migrating from GdkColor to GdkRGBA yet");
+        return;
+    }
+
     /* Use the old property format */
     g_snprintf(buf, sizeof(buf), "%smonitor%d/",
                workspace->priv->property_prefix, monitor);
@@ -481,6 +505,12 @@ xfce_workspace_migrate_backdrop_second_color(XfceWorkspace *workspace,
 
     TRACE("entering");
 
+    /* TODO: migrate from color1 (and older) to the new rgba1 */
+    if(TRUE) {
+        TRACE("warning: we aren't migrating from GdkColor to GdkRGBA yet");
+        return;
+    }
+
     /* Use the old property format */
     g_snprintf(buf, sizeof(buf), "%smonitor%d/",
                workspace->priv->property_prefix, monitor);
@@ -598,11 +628,19 @@ xfce_workspace_connect_backdrop_settings(XfceWorkspace *workspace,
     XfconfChannel *channel = workspace->priv->channel;
     char buf[1024];
     gint pp_len;
+#if 0 /* GTK_CHECK_VERSION (3, 22, 0) */
+    GdkDisplay *display;
+#endif
     gchar *monitor_name = NULL;
 
     TRACE("entering");
 
+#if 0 /* GTK_CHECK_VERSION (3, 22, 0) */
+    display = gdk_display_manager_get_default_display(gdk_display_manager_get());
+    monitor_name = g_strdup(gdk_monitor_get_model(gdk_display_get_monitor(display, monitor)));
+#else
     monitor_name = gdk_screen_get_monitor_plug_name(workspace->priv->gscreen, monitor);
+#endif
 
     if(monitor_name == NULL) {
         g_snprintf(buf, sizeof(buf), "%smonitor%d/workspace%d/",
@@ -624,19 +662,19 @@ xfce_workspace_connect_backdrop_settings(XfceWorkspace *workspace,
                            G_OBJECT(backdrop), "color-style");
 
     buf[pp_len] = 0;
-    g_strlcat(buf, "color1", sizeof(buf));
+    g_strlcat(buf, "rgba1", sizeof(buf));
     if(!xfconf_channel_has_property(channel, buf)) {
         xfce_workspace_migrate_backdrop_first_color(workspace, backdrop, monitor);
     }
-    workspace->priv->first_color_id[monitor] = xfconf_g_property_bind_gdkcolor(channel, buf,
+    workspace->priv->first_color_id[monitor] = xfconf_g_property_bind_gdkrgba(channel, buf,
                                                             G_OBJECT(backdrop), "first-color");
 
     buf[pp_len] = 0;
-    g_strlcat(buf, "color2", sizeof(buf));
+    g_strlcat(buf, "rgba2", sizeof(buf));
     if(!xfconf_channel_has_property(channel, buf)) {
         xfce_workspace_migrate_backdrop_second_color(workspace, backdrop, monitor);
     }
-    workspace->priv->second_color_id[monitor] = xfconf_g_property_bind_gdkcolor(channel, buf,
+    workspace->priv->second_color_id[monitor] = xfconf_g_property_bind_gdkrgba(channel, buf,
                                                             G_OBJECT(backdrop), "second-color");
 
     buf[pp_len] = 0;
@@ -697,7 +735,11 @@ xfce_workspace_remove_backdrops(XfceWorkspace *workspace)
 
     g_return_if_fail(XFCE_IS_WORKSPACE(workspace));
 
+#if 0 /* GTK_CHECK_VERSION (3, 22, 0) */
+    n_monitors = gdk_display_get_n_monitors(gdk_screen_get_display(workspace->priv->gscreen));
+#else
     n_monitors = gdk_screen_get_n_monitors(workspace->priv->gscreen);
+#endif
 
     for(i = 0; i < n_monitors && i < workspace->priv->nbackdrops; ++i) {
         xfce_workspace_disconnect_backdrop_settings(workspace,
