@@ -51,14 +51,14 @@ struct _XfdesktopIconViewManagerIface
     gboolean (*drag_drop)(XfdesktopIconViewManager *manager,
                           XfdesktopIcon *drop_icon,
                           GdkDragContext *context,
-                          guint16 row,
-                          guint16 col,
+                          gint16 row,
+                          gint16 col,
                           guint time_);
     void (*drag_data_received)(XfdesktopIconViewManager *manager,
                                XfdesktopIcon *drop_icon,
                                GdkDragContext *context,
-                               guint16 row,
-                               guint16 col,
+                               gint16 row,
+                               gint16 col,
                                GtkSelectionData *data,
                                guint info,
                                guint time_);
@@ -87,14 +87,14 @@ void xfdesktop_icon_view_manager_fini(XfdesktopIconViewManager *manager);
 gboolean xfdesktop_icon_view_manager_drag_drop(XfdesktopIconViewManager *manager,
                                                XfdesktopIcon *drop_icon,
                                                GdkDragContext *context,
-                                               guint16 row,
-                                               guint16 col,
+                                               gint16 row,
+                                               gint16 col,
                                                guint time_);
 void xfdesktop_icon_view_manager_drag_data_received(XfdesktopIconViewManager *manager,
                                                     XfdesktopIcon *drop_icon,
                                                     GdkDragContext *context,
-                                                    guint16 row,
-                                                    guint16 col,
+                                                    gint16 row,
+                                                    gint16 col,
                                                     GtkSelectionData *data,
                                                     guint info,
                                                     guint time_);
