@@ -520,15 +520,16 @@ xfdesktop_volume_icon_eject_finish(GObject *object,
 
             /* display an error dialog to inform the user */
             xfce_message_dialog(toplevel ? GTK_WINDOW(toplevel) : NULL,
-                                _("Eject Failed"), GTK_STOCK_DIALOG_ERROR, 
+                                _("Eject Failed"), "dialog-error", 
                                 primary, error->message,
-                                GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT, NULL);
+                                XFCE_BUTTON_TYPE_MIXED, "window-close", _("_Close"), GTK_RESPONSE_ACCEPT,
+                                NULL);
 
             g_free(primary);
             g_free(volume_name);
         }
 
-        g_error_free(error);
+        g_clear_error(&error);
     }
 
 #ifdef HAVE_LIBNOTIFY
@@ -565,15 +566,16 @@ xfdesktop_volume_icon_unmount_finish(GObject *object,
 
             /* display an error dialog to inform the user */
             xfce_message_dialog(toplevel ? GTK_WINDOW(toplevel) : NULL,
-                                _("Eject Failed"), GTK_STOCK_DIALOG_ERROR, 
+                                _("Eject Failed"), "dialog-error", 
                                 primary, error->message,
-                                GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT, NULL);
+                                XFCE_BUTTON_TYPE_MIXED, "window-close", _("_Close"), GTK_RESPONSE_ACCEPT,
+                                NULL);
 
             g_free(primary);
             g_free(mount_name);
         }
 
-        g_error_free(error);
+        g_clear_error(&error);
     }
 
 #ifdef HAVE_LIBNOTIFY
@@ -600,14 +602,15 @@ xfdesktop_volume_icon_mount_finish(GObject *object,
             gchar *primary = g_markup_printf_escaped(_("Failed to mount \"%s\""),
                                                      volume_name);
             xfce_message_dialog(toplevel ? GTK_WINDOW(toplevel) : NULL,
-                                _("Mount Failed"), GTK_STOCK_DIALOG_ERROR, 
+                                _("Mount Failed"), "dialog-error", 
                                 primary, error->message,
-                                GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT, NULL);
+                                XFCE_BUTTON_TYPE_MIXED, "window-close", _("_Close"), GTK_RESPONSE_ACCEPT,
+                                NULL);
             g_free(primary);
             g_free(volume_name);
         }
         
-        g_error_free(error);
+        g_clear_error(&error);
     } else {
         GMount *mount = g_volume_get_mount(volume);
         GFile *file = NULL;
@@ -782,9 +785,7 @@ xfdesktop_volume_icon_add_context_menu_option(XfdesktopIcon *icon,
     GtkWidget *mi, *img;
 
     img = gtk_image_new_from_icon_name(icon_name, GTK_ICON_SIZE_MENU);
-    gtk_widget_show(img);
-    mi = gtk_image_menu_item_new_with_mnemonic(icon_label);
-    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), img);
+    mi = xfdesktop_menu_create_menu_item_with_mnemonic(icon_label, img);
     gtk_widget_show(mi);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
     if(callback != NULL) {
@@ -806,12 +807,10 @@ xfdesktop_volume_icon_populate_context_menu(XfdesktopIcon *icon,
     GMount *mount;
     const gchar *icon_name, *icon_label;
 
-    icon_name = GTK_STOCK_OPEN;
+    icon_name = "document-open";
 
-    img = gtk_image_new_from_stock(GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU);
-    gtk_widget_show(img);
-    mi = gtk_image_menu_item_new_with_mnemonic(_("_Open"));
-    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), img);
+    img = gtk_image_new_from_icon_name("document-open", GTK_ICON_SIZE_MENU);
+    mi = xfdesktop_menu_create_menu_item_with_mnemonic(_("_Open"), img);
     gtk_widget_show(mi);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
     g_signal_connect_swapped(G_OBJECT(mi), "activate",
@@ -831,14 +830,14 @@ xfdesktop_volume_icon_populate_context_menu(XfdesktopIcon *icon,
     }
 
     if(mount && g_mount_can_unmount(mount)) {
-        icon_name = NULL;
+        icon_name = "drive-removable-media";
         icon_label = _("_Unmount Volume");
         xfdesktop_volume_icon_add_context_menu_option(icon, icon_name, icon_label,
                         menu, G_CALLBACK(xfdesktop_volume_icon_menu_unmount));
     }
 
     if(!mount && g_volume_can_mount(volume)) {
-        icon_name = NULL;
+        icon_name = "drive-removable-media";
         icon_label = _("_Mount Volume");
         xfdesktop_volume_icon_add_context_menu_option(icon, icon_name, icon_label,
                         menu, G_CALLBACK(xfdesktop_volume_icon_menu_mount));
@@ -851,7 +850,7 @@ xfdesktop_volume_icon_populate_context_menu(XfdesktopIcon *icon,
     gtk_widget_show(mi);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
 
-    icon_name = GTK_STOCK_PROPERTIES;
+    icon_name = "document-properties";
     icon_label = _("P_roperties...");
 
     if(!volume_icon->priv->file_info)
